/*  ProMoIJ_M1_Cropping is an ImageJ macro that allows reducing the number of image channels, 
 *   z-slices and/or time-frames, and cropping cells to generate smaller images for further processing
    
    Copyright (C) 2017  Jorge Valero Gómez-Lobo, Iñaki Paris & Amanda Sierra.

  	ProMoIJ_M1_Cropping is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ProMoIJ_M1_Cropping is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//This macro has been developed by Iñaki Paris, Amanda Sierra & Jorge Valero (jorge.valero@achucarro.org). 
//If you have any doubt about how to use it, please contact us.

//License
Dialog.create("GNU GPL License");
Dialog.addMessage("ProMoIJ_M1_Cropping (C) 2017 J. Valero, I. Paris & A. Sierra.");
Dialog.setInsets(10, 20, 0);
Dialog.addMessage("ProMoIJ_M1_Cropping comes with ABSOLUTELY NO WARRANTY.");
Dialog.setInsets(0, 20, 0);
Dialog.addMessage("This is free software, and you are welcome to redistribute it under certain conditions. Click on the Help button for details.");
Dialog.addHelp("http://www.achucarro.org/downloads");
Dialog.show();

setBatchMode(true);

/*Selects the folder that contains the original image files to be cropped. Image files will be removed from the original folder 
 * and transferred as processed images to a second folder. 
 *any unsused images will be tranferred to a third folder for non-processed image files.
 *ROIs will be saved as originalname.zip in a fourth folder and cropped images as originalname_Crop.tif in a fifth folder. 
*/
dir=getDirectory("Please, select the M1 input folder: Original");
dirOrig=getDirectory("Please, select the M1 output folder for processed images: M1 Processed");
dirNonPro=getDirectory("Please, select the M1 output folder for discarded images: M1 Discarded");
dirRoi=getDirectory("Please, select the M1 outup folder for ROIs: M1 Cropping ROIs");
dirreg=getDirectory("Please, select the M1 output folder for cropped images: M1 Cropped");


//Opens an image file and obtains image dimensions
images=getFileList(dir);
v=0;
while(endsWith(images[v], "/")) v++;
run("Bio-Formats Importer", "open=["+dir+images[v]+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
getDimensions(width0, height0, channels0, slices0, frames0);
selectWindow(getTitle());
close();
chan0=newArray(channels0);
for (i=1; i<=channels0; i++) chan0[i-1]=i;

/*The user selects the channel, the z-slices, and the time frames that will be maintained after cropping:
 * M1.1 Crop cells: Activates cell cropping
 * M1.2 Channel: Channel used to select analyze the cells 
 * M1.3 Select Z-slices: Activates selection of Z-slices
 * M1.4 First Z-slice: Selects the first Z-slice to be used
 * M1.5 Last Z-slice: Selects the last Z-slice to be used
 * M1.6 Select time frames: Activates selections of time frames
 * M1.7 First time frame: Selects the first time frame to be used
 * M1.8 Last time frame: Selects the last time frame to be used
 */
Dialog.create("ProMoIJ Macro1 Cropping parameters");
Dialog.addCheckbox("M1.1 Crop cells", false);
Dialog.addChoice("M1.2 Channel", chan0);
Dialog.addCheckbox("M1.3 Select Z-slices", false);
Dialog.addSlider("M1.4 First Z-slice", 1, slices0, 1);
Dialog.addSlider("M1.5 Last Z-slice", 1, slices0, slices0);
Dialog.addCheckbox("M1.6 Select time frames", false);
Dialog.addSlider("M1.7 First time frame", 1, frames0, 1);
Dialog.addSlider("M1.8 Last time frame", 1, frames0, frames0);
Dialog.addHelp("http://www.achucarro.org/downloads");
Dialog.show()

cropcells=Dialog.getCheckbox();
chanselected=Dialog.getChoice();
changeslice=Dialog.getCheckbox();
firstslice=Dialog.getNumber();
lastslice=Dialog.getNumber();
changeframes=Dialog.getCheckbox();
firstframe=Dialog.getNumber();
lastframe=Dialog.getNumber();

//Loop that opens all image files and crops them.
for (v=0; v<images.length; v++){
	print("Processing image file "+v+1+" of "+images.length);
	if (endsWith(images[v], "/")) print("NO"); 
	else work(dir+images[v]);
	for (gc=0; gc<5; gc++) run("Collect Garbage");	
} 
print("JOB DONE");

//The function "work" will open the image files, make a maximum intensity projection, and then a time color-coded projection.
function work(imagepath){
	if (cropcells==true) setBatchMode(false);
	
 	run("Bio-Formats Importer", "open=["+imagepath+"] color_mode=Grayscale open_files view=Hyperstack stack_order=XYCZT");
	getDimensions(width1, height1, channels1, slices1, frames1);
	name=getTitle();
	namenoext=File.nameWithoutExtension();
	if (changeslice==false || lastslice>slices1){
		firstslice=1;
		lastslice=slices1;
	}
	if (changeframes==false || lastframe>frames1){
		firstframe=1;
		lastframe=frames1;
	}
	if (slices0>1) run("Duplicate...", "title=temporal duplicate channels="+chanselected+" slices="+firstslice+"-"+lastslice+" frames="+firstframe+"-"+lastframe);
	else run("Duplicate...", "title=temporal duplicate range="+firstframe+"-"+lastframe);
	
	selectWindow("temporal");
	rename(namenoext);
	getDimensions(width2, height2, channels2, slices2, frames2);
	selectWindow(name);
	close();
	if (cropcells==true) cellcropper();
	else {
		selectWindow(namenoext);
		saveAs("Tiff", dirreg+namenoext+"_Crop.tif");
		close();
		File.rename(dir+images[v], dirOrig+images[v]);
	}
	
}

//The function cellcropper allows cell selection
function cellcropper(){
	if(slices0>1) run("Z Project...", "projection=[Max Intensity] all");
	run("Temporal-Color Code", "lut=[Red/Green] start=1 end="+frames2+" create");
	selectWindow("color time scale");
	close();
	
	//Allows the user checking the image and deciding whether it will be further used
	waitForUser("Please, check the image and click OK");
	usar=getBoolean("Do you want to select cells from this image?");
		if (usar==true){

			do{
				cont=true;
				setTool("polygon");
				waitForUser("Please, select single cells by drawing ROIs around them using the Polygon tool from Fiji toolbar. \n Then add each one to the ROI Manager pressing the letter \"t\"");
				roiManager("Show All");
				cont=getBoolean("Do you want to select another cell?");
			}while(cont==true);
			 selectWindow("MAX_colored");
			close();
			if(slices0>1)  {
				selectWindow("MAX_"+namenoext);
				close();
			}
			
			for (gc=0; gc<5; gc++) run("Collect Garbage");	
			setBatchMode(true);
			crop();
			setBatchMode(false);
			roiManager("Deselect");
			roiManager("Save", dirRoi+namenoext+".zip");
			roiManager("Delete");
			File.rename(dir+images[v], dirOrig+images[v]);
		}
		else{
			File.rename(dir+images[v], dirNonPro+images[v]);
			selectWindow("MAX_colored");
			close();
			selectWindow("MAX_"+namenoext);
			close();
			selectWindow(namenoext);
			close();
		}
	for (gc=0; gc<5; gc++) run("Collect Garbage");	
}
		
	


//The function "crop" crops the image using pre-defined ROIs and the "cropping" function
function crop(){
	
	selectWindow(namenoext);				
	//Loop that selects the ROIs, locates them on the image, crops their bounding box, and saves cropped images
	cuantos=roiManager("count");
	for(buf=0; buf<cuantos; buf++){
		cropping(namenoext, "crop", buf, false);
		selectWindow("crop");
		saveAs("Tiff", dirreg+namenoext+buf+"_Crop.tif");
		close();
		run("Select None");	
	}
	selectWindow(namenoext);
	close();
}

//The function "cropping" crops the image
function cropping(imagewin, imageout, rselected, optionclear){
	selectWindow(imagewin);
	roiManager("Select", rselected);
	run("Duplicate...", "title="+imageout+" duplicate");
	if (optionclear==true){
		run("Clear Outside", "stack"); 
	}
	run("Select None");
}

